// ----------------------------------------------------------------------------
//
//  Copyright (C) 2015-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <string.h>
#include "ambixfm3.h"


const char * const Ladspa_Ambixtofuma3::portnames [NPORT] = 
{
    "in.0", "in.1", "in.2", "in.3", "in.4", "in.5", "in.6", "in.7", 
    "in.8", "in.9", "in.10", "in.11", "in.12", "in.13", "in.14", "in.15", 
    "out.W", "out.X", "out.Y", "out.Z", "out.R", "out.S", "out.T", "out.U", 
    "out.V", "out.K", "out.L", "out.M", "out.N", "out.O", "out.P", "out.Q"
};


LADSPA_Handle Ladspa_Ambixtofuma3::create (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
    return new Ladspa_Ambixtofuma3 (rate);
}


void Ladspa_Ambixtofuma3::setport (unsigned long port, LADSPA_Data *data)
{
    _port [port] = data;
}


void Ladspa_Ambixtofuma3::active (bool act)
{
}


void Ladspa_Ambixtofuma3::runproc (unsigned long len, bool add)
{
    unsigned int  i;
    float         v;
    
    // This could be simpler and more efficient if Ardour
    // wouldn't require in-place-safe processing.

    for (i = 0; i < len; i++)
    {
	_port [OUT +  0][i] =  0.707107f * _port [INP +  0][i];
	_port [OUT + 11][i] =  1.185854f * _port [INP + 11][i];
    }	
    for (i = 0; i < len; i++)
    {
	v = _port [INP + 1][i];
	_port [OUT + 1][i] =  _port [INP + 3][i];
	_port [OUT + 3][i] =  _port [INP + 2][i];
	_port [OUT + 2][i] =  v;
    }	
    for (i = 0; i < len; i++)
    {
	v =  _port [INP + 6][i];
	_port [OUT + 6][i] =  1.154701f * _port [INP + 5][i];
	_port [OUT + 5][i] =  1.154701f * _port [INP + 7][i];
	_port [OUT + 7][i] =  1.154701f * _port [INP + 8][i];
	_port [OUT + 8][i] =  1.154701f * _port [INP + 4][i];
	_port [OUT + 4][i] =  v;
    }	
    for (i = 0; i < len; i++)
    {
	v = _port [INP +  12][i] ;
	_port [OUT + 12][i] = 1.341641f * _port [INP + 14][i];
	_port [OUT + 14][i] = 1.264911f * _port [INP + 15][i];
	_port [OUT + 15][i] = 1.264911f * _port [INP +  9][i];
	_port [OUT +  9][i] = v;
    }	
    for (i = 0; i < len; i++)
    {
	v = _port [INP + 10][i];
	_port [OUT + 10][i] = 1.185854f * _port [INP + 13][i];
	_port [OUT + 13][i] = 1.341641f * v;
    }	
}


// ----------------------------------------------------------------------------


const char * const Ladspa_Fumatoambix3::portnames [NPORT] = 
{
    "in.W", "in.X", "in.Y", "in.Z", "in.R", "in.S", "in.T", "in.U", 
    "in.V", "in.K", "in.L", "in.M", "in.N", "in.O", "in.P", "in.Q", 
    "out.0", "out.1", "out.2", "out.3", "out.4", "out.5", "out.6", "out.7", 
    "out.8", "out.9", "out.10", "out.11", "out.12", "out.13", "out.14", "out.15"
};


LADSPA_Handle Ladspa_Fumatoambix3::create (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
    return new Ladspa_Fumatoambix3 (rate);
}


void Ladspa_Fumatoambix3::setport (unsigned long port, LADSPA_Data *data)
{
    _port [port] = data;
}


void Ladspa_Fumatoambix3::active (bool act)
{
}


void Ladspa_Fumatoambix3::runproc (unsigned long len, bool add)
{
    unsigned int  i;
    float         v;
    
    // This could be simpler and more efficient if Ardour
    // wouldn't require in-place-safe processing.

    for (i = 0; i < len; i++)
    {
	_port [OUT +  0][i] =  1.414214f * _port [INP +  0][i];
	_port [OUT + 11][i] =  0.843274f * _port [INP + 11][i];
    }	
    for (i = 0; i < len; i++)
    {
	v = _port [INP + 2][i];
	_port [OUT + 2][i] =  _port [INP + 3][i];
	_port [OUT + 3][i] =  _port [INP + 1][i];
	_port [OUT + 1][i] =  v;
    }	
    for (i = 0; i < len; i++)
    {
	v =  _port [INP + 4][i];
	_port [OUT + 4][i] =  0.866025f * _port [INP + 8][i];
	_port [OUT + 8][i] =  0.866025f * _port [INP + 7][i];
	_port [OUT + 7][i] =  0.866025f * _port [INP + 5][i];
	_port [OUT + 5][i] =  0.866025f * _port [INP + 6][i];
	_port [OUT + 6][i] =  v;
    }	
    for (i = 0; i < len; i++)
    {
	v = _port [INP +  9][i] ;
	_port [OUT +  9][i] = 0.790569f * _port [INP + 15][i];
	_port [OUT + 15][i] = 0.790569f * _port [INP + 14][i];
	_port [OUT + 14][i] = 0.745356f * _port [INP + 12][i];
	_port [OUT + 12][i] = v;
    }	
    for (i = 0; i < len; i++)
    {
	v = _port [INP + 13][i];
	_port [OUT + 13][i] =  0.843274f * _port [INP + 10][i];
	_port [OUT + 10][i] =  0.745356f * v;
    }	
}


// ----------------------------------------------------------------------------
