// ----------------------------------------------------------------------------
//
//  Copyright (C) 2015-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <string.h>
#include <math.h>
#include "convert3.h"


const LADSPA_PortDescriptor Ladspa_Hoa3convert::portdescr [NPORT] = 
{
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO
};


const LADSPA_PortRangeHint Ladspa_Hoa3convert::porthints [NPORT] = 
{
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 }
};


const char * const Ladspa_Hoa3convert::portnames [NPORT] = 
{
    "in.0", "in.1", "in.2", "in.3", "in.4", "in.5", "in.6", "in.7", 
    "in.8", "in.9", "in.10", "in.11", "in.12", "in.13", "in.14", "in.15", 
    "out.0", "out.1", "out.2", "out.3", "out.4", "out.5", "out.6", "out.7", 
    "out.8", "out.9", "out.10", "out.11", "out.12", "out.13", "out.14", "out.15"
};


LADSPA_Handle Ladspa_Hoa3convert::create_sn3d_to_n3d (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
    return new Ladspa_Hoa3convert (rate, false);
}


LADSPA_Handle Ladspa_Hoa3convert::create_n3d_to_sn3d (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
    return new Ladspa_Hoa3convert (rate, true);
}


Ladspa_Hoa3convert::Ladspa_Hoa3convert (unsigned long fsam, bool inverse) :
    LadspaPlugin (fsam),
    _inverse (inverse)
{
}


void Ladspa_Hoa3convert::setport (unsigned long port, LADSPA_Data *data)
{
    _port [port] = data;
}


void Ladspa_Hoa3convert::active (bool act)
{
}


void Ladspa_Hoa3convert::runproc (unsigned long len, bool add)
{
    unsigned int i, k;
    float  g;
    float  *inp, *out;

    memcpy (_port [OUT], _port [INP], len * sizeof (float));
    g = sqrtf (3.0f);
    if (_inverse) g = 1.0f / g;
    for (i = 1; i < 4; i++)
    {
	inp = _port [INP + i];
	out = _port [OUT + i];
        for (k = 0; k < len; k++) out [k] = g * inp [k];
    }
    g = sqrtf (5.0f);
    if (_inverse) g = 1.0f / g;
    for (i = 4; i < 9; i++)
    {
	inp = _port [INP + i];
	out = _port [OUT + i];
        for (k = 0; k < len; k++) out [k] = g * inp [k];
    }
    g = sqrtf (7.0f);
    if (_inverse) g = 1.0f / g;
    for (i = 9; i < 16; i++)
    {
	inp = _port [INP + i];
	out = _port [OUT + i];
        for (k = 0; k < len; k++) out [k] = g * inp [k];
    }
}

