// ----------------------------------------------------------------------------
//
//  Copyright (C) 2015-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <string.h>
#include <math.h>
#include "sn3dhtop3.h"


// Convert an SN3D/ACN horizontal signal set to periphonic.


const LADSPA_PortDescriptor Ladspa_Sn3dhtop3::portdescr [NPORT] = 
{
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO
};


const char * const Ladspa_Sn3dhtop3::portnames [NPORT] = 
{
    "in.0", "in.1", "in.3", "in.4", "in.8", "in.9","in.15",
    "out.0", "out.1", "out.2", "out.3", "out.4", "out.5", "out.6", "out.7", 
    "out.8", "out.9", "out.10", "out.11", "out.12", "out.13", "out.14", "out.15" 
};


const LADSPA_PortRangeHint Ladspa_Sn3dhtop3::porthints [NPORT] = 
{
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 }
};


LADSPA_Handle Ladspa_Sn3dhtop3::create (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
    return new Ladspa_Sn3dhtop3 (rate);
}


void Ladspa_Sn3dhtop3::setport (unsigned long port, LADSPA_Data *data)
{
    _port [port] = data;
}


void Ladspa_Sn3dhtop3::active (bool act)
{
}


void Ladspa_Sn3dhtop3::runproc (unsigned long len, bool add)
{
    unsigned int  i, k, n;
    float  *p;
    
    for (k = n = 0; k < len; k += n)
    {
	n = len - k;
	if (n > BSIZE) n = BSIZE;

	// This could be a lot simpler if Ardour wouldn't
	// require in-place-safe processing even in cases
	// like this one where it doesn't make any sense.
	
	memcpy (_buff0, _port [INP + 0] + k, n * sizeof (float));
	memcpy (_buff1, _port [INP + 1] + k, n * sizeof (float));
	memcpy (_buff2, _port [INP + 2] + k, n * sizeof (float));
	memcpy (_buff3, _port [INP + 3] + k, n * sizeof (float));
	memcpy (_buff4, _port [INP + 4] + k, n * sizeof (float));
	memcpy (_buff5, _port [INP + 5] + k, n * sizeof (float));
	memcpy (_buff6, _port [INP + 6] + k, n * sizeof (float));

        memset (_port [OUT +  2] + k, 0, n * sizeof (float));
        memset (_port [OUT +  5] + k, 0, n * sizeof (float));
        memset (_port [OUT +  7] + k, 0, n * sizeof (float));
        memset (_port [OUT + 10] + k, 0, n * sizeof (float));
        memset (_port [OUT + 12] + k, 0, n * sizeof (float));
        memset (_port [OUT + 14] + k, 0, n * sizeof (float));
	
        memcpy (_port [OUT +  0] + k, _buff0, n * sizeof (float));
        memcpy (_port [OUT +  1] + k, _buff1, n * sizeof (float));
        memcpy (_port [OUT +  3] + k, _buff2, n * sizeof (float));
        memcpy (_port [OUT +  4] + k, _buff3, n * sizeof (float));
        memcpy (_port [OUT +  8] + k, _buff4, n * sizeof (float));
        memcpy (_port [OUT +  9] + k, _buff5, n * sizeof (float));
        memcpy (_port [OUT + 15] + k, _buff6, n * sizeof (float));

	p = _port [OUT + 6] + k;
	for (i = 0; i < n; i++) p [i] = -0.5f * _buff0 [i];
	p = _port [OUT + 11] + k;
	for (i = 0; i < n; i++) p [i] = -0.6123724f * _buff1 [i];
	p = _port [OUT + 13] + k;
	for (i = 0; i < n; i++) p [i] = -0.6123724f * _buff2 [i];
    }
}

