// ----------------------------------------------------------------------------
//
//  Copyright (C) 2015-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <string.h>
#include <math.h>
#include "zrotator3.h"


#define DEG2RAD 0.01745329f


const LADSPA_PortDescriptor Ladspa_Zrotator3::portdescr [NPORT] = 
{
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL
};


const char * const Ladspa_Zrotator3::portnames [NPORT] = 
{
    "in.0", "in.1", "in.2", "in.3", "in.4", "in.5", "in.6", "in.7", 
    "in.8", "in.9", "in.10", "in.11", "in.12", "in.13", "in.14", "in.15", 
    "out.0", "out.1", "out.2", "out.3", "out.4", "out.5", "out.6", "out.7", 
    "out.8", "out.9", "out.10", "out.11", "out.12", "out.13", "out.14", "out.15", 
    "Az"
};


const LADSPA_PortRangeHint Ladspa_Zrotator3::porthints [NPORT] = 
{
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0, -180, 180 }
};


LADSPA_Handle Ladspa_Zrotator3::create (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
    return new Ladspa_Zrotator3 (rate);
}


void Ladspa_Zrotator3::setport (unsigned long port, LADSPA_Data *data)
{
    _port [port] = data;
}


void Ladspa_Zrotator3::active (bool act)
{
    if (act) calcpar (0.0f);
}


void Ladspa_Zrotator3::calcpar (float az)
{
    az *= DEG2RAD;
    _c1 = cosf (az);
    _s1 = sinf (az);
    _c2 = _c1 * _c1 - _s1 * _s1;
    _s2 = 2 * _c1 * _s1;
    _c3 = _c2 * _c1 - _s2 * _s1;
    _s3 = _c2 * _s1 + _c1 * _s2;
}


void Ladspa_Zrotator3::runproc (unsigned long len, bool add)
{
    int    k;
    float  c1, s1, c2, s2, c3, s3;
    float  x, y, dc, ds;
    float  *inp1, *inp2, *inp3, *inp4, *inp5, *inp6;
    float  *out1, *out2, *out3, *out4, *out5, *out6;

    c1 = _c1;
    s1 = _s1;
    c2 = _c2;
    s2 = _s2;
    c3 = _c3;
    s3 = _s3; 
    calcpar (_port [CTL_AZ][0]);

    memcpy (_port [OUT +  0], _port [INP +  0], len * sizeof (LADSPA_Data));
    memcpy (_port [OUT +  2], _port [INP +  2], len * sizeof (LADSPA_Data));
    memcpy (_port [OUT +  6], _port [INP +  6], len * sizeof (LADSPA_Data));
    memcpy (_port [OUT + 12], _port [INP + 12], len * sizeof (LADSPA_Data));

    inp1 = _port [INP + 3];
    inp2 = _port [INP + 1];
    inp3 = _port [INP + 7];
    inp4 = _port [INP + 5];
    inp5 = _port [INP + 13];
    inp6 = _port [INP + 11];
    out1 = _port [OUT + 3];
    out2 = _port [OUT + 1];
    out3 = _port [OUT + 7];
    out4 = _port [OUT + 5];
    out5 = _port [OUT + 13];
    out6 = _port [OUT + 11];
    k = len;
    dc = (_c1 - c1) / len;
    ds = (_s1 - s1) / len;
    while (k--)
    {
	c1 += dc;
	s1 += ds;
        x = *inp1++;
        y = *inp2++; 
        *out1++ = c1 * x + s1 * y;
        *out2++ = c1 * y - s1 * x;
        x = *inp3++;
        y = *inp4++; 
        *out3++ = c1 * x + s1 * y;
        *out4++ = c1 * y - s1 * x;
        x = *inp5++;
        y = *inp6++; 
        *out5++ = c1 * x + s1 * y;
        *out6++ = c1 * y - s1 * x;
    }

    inp1 = _port [INP + 8];
    inp2 = _port [INP + 4];
    inp3 = _port [INP + 14];
    inp4 = _port [INP + 10];
    out1 = _port [OUT + 8];
    out2 = _port [OUT + 4];
    out3 = _port [OUT + 14];
    out4 = _port [OUT + 10];
    k = len;
    dc = (_c2 - c2) / len;
    ds = (_s2 - s2) / len;
    while (k--)
    {
	c2 += dc;
	s2 += ds;
        x = *inp1++;
        y = *inp2++; 
        *out1++ = c2 * x + s2 * y;
        *out2++ = c2 * y - s2 * x;
        x = *inp3++;
        y = *inp4++; 
        *out3++ = c2 * x + s2 * y;
        *out4++ = c2 * y - s2 * x;
    }

    inp1 = _port [INP + 15];
    inp2 = _port [INP + 9];
    out1 = _port [OUT + 15];
    out2 = _port [OUT + 9];
    k = len;
    dc = (_c3 - c3) / len;
    ds = (_s3 - s3) / len;
    while (k--)
    {
	c3 += dc;
	s3 += ds;
        x = *inp1++;
        y = *inp2++; 
        *out1++ = c3 * x + s3 * y;
        *out2++ = c3 * y - s3 * x;
    }
}
